%define anolis_release 2
%bcond_with replace_coreutils
%bcond_without replace_fipscheck
%bcond_without replace_hmaccalc
%bcond_without test_package
%bcond_with cppcheck
%bcond_without test
%bcond_without fuzz_test
%bcond_without doc
%bcond_without clang_sa

%global min_kernel_ver    4.10.0

%if %{lua:print(rpm.vercmp('99.0.0', posix.uname('%r')));} >= 0
%global with_sysctl_tweak 1
%else
%global with_sysctl_tweak 0
%endif

%if %{with_sysctl_tweak}
%global sysctl_prio       50
%global sysctl_optmem_max 81920
%global distroname_ext    anolis
%endif

%if %{with replace_coreutils}
%global coreutils_evr     8.29-1%{?dist}
%endif

%if %{with replace_fipscheck}
%global fipscheck_evr     1.5.0-9
%endif

%if %{with replace_hmaccalc}
%global hmaccalc_evr      0.9.14-10%{?dist}
%endif

%global apps_hmaccalc sha1hmac sha224hmac sha256hmac sha384hmac sha512hmac sm3hmac
%global apps_fipscheck sha1sum sha224sum sha256sum sha384sum sha512sum md5sum sm3sum fipscheck fipshmac

%if %{lua:print(rpm.vercmp(posix.uname('%r'), '3.19'));} >= 0
%global sha512hmac bin/kcapi-hasher -n sha512hmac
%global fipshmac   bin/kcapi-hasher -n fipshmac
%else
%global sha512hmac bash %{SOURCE2}
%global fipshmac   bash %{SOURCE3}
%endif

%define __spec_install_post                                      \
%{?__debug_package:%{__debug_install_post}}                      \
%{__arch_install_post}                                           \
%{__os_install_post}                                             \
bin_path=%{buildroot}%{_bindir}                                  \
lib_path=%{buildroot}/%{_lib}                                    \
for app in %{apps_hmaccalc}; do                                  \
  test -e "$bin_path"/$app || continue                           \
  { %sha512hmac "$bin_path"/$app || exit 1; }                    \\\
    | cut -f 1 -d ' ' >"$lib_path"/hmaccalc/$app.hmac            \
done                                                             \
for app in %{apps_fipscheck}; do                                 \
  test -e "$bin_path"/$app || continue                           \
  %fipshmac -d "$lib_path"/fipscheck "$bin_path"/$app || exit 1  \
done                                                             \
%{_bindir}/hardlink -cfv %{buildroot}%{_bindir}                  \
%fipshmac -d "$lib_path"/fipscheck                               \\\
  "$lib_path"/libkcapi.so.%{version} || exit 1                   \
maj=$(echo %{version} | cut -d. -f1)                             \
ln -s libkcapi.so.%{version}.hmac                                \\\
  "$lib_path"/fipscheck/libkcapi.so.$maj.hmac            \
%{nil}

Name:           libkcapi
Version:        1.4.0
Release:        %{anolis_release}%{?dist}
Summary:        User space interface to the Linux Kernel Crypto API

License:        BSD or GPLv2
URL:            https://www.chronox.de/%{name}.html
Source0:        https://www.chronox.de/%{name}/%{name}-%{version}.tar.xz
Source1:        https://www.chronox.de/%{name}/%{name}-%{version}.tar.xz.asc
Source2:        sha512hmac-openssl.sh
Source3:        fipshmac-openssl.sh
%if %{with_sysctl_tweak}
Source4:        README.%{distroname_ext}
Source5:        %{sysctl_prio}-%{name}-optmem_max.conf
%endif

BuildRequires:  bash, gcc > 12.0, make, libtool
BuildRequires:  coreutils, openssl, perl-interpreter
BuildRequires:  hardlink
BuildRequires:  kernel-headers >= 4.10.0
BuildRequires:  systemd
BuildRequires:  xmlto
%if %{with doc}
BuildRequires:  docbook-utils-pdf
%endif
%if %{with clang_sa}
BuildRequires:  clang
%endif
%if %{with cppcheck}
BuildRequires:  cppcheck >= 2.4
%endif

Requires:       systemd
Obsoletes:      %{name}-replacements <= %{version}-%{release}

%description
libkcapi allows user-space to access the Linux kernel crypto API.

This library uses the netlink interface and exports easy to use APIs
so that a developer does not need to consider the low-level netlink
It uses this Netlink interface and exports easy to use APIs
so that a developer does not need to consider the low-level Netlink
interface handling.

The library does not implement any cipher algorithms.  All consumer
requests are sent to the kernel for processing.  Results from the
kernel crypto API are returned to the consumer via the library API.
The library does not implement any cipher algorithms. All consumer
requests are sent to the kernel for processing. Results from the kernel
crypto API are returned to the consumer via the library API.

The kernel interface and therefore this library can be used by
unprivileged processes.

The focus during the development of this library is put on speed.
This library does not perform any memcpy for processing the cryptographic
data! The library uses scatter / gather lists to eliminate the need for
moving data around in memory.


%package        devel
Summary:        Development files for the %{name} package
Requires:       %{name} == %{version}-%{release}

%description    devel
Header files for applications that use %{name}.


%if %{with replace_coreutils}
%package        checksum
Summary:        Drop-in replacement for *sum utils provided by the %{name} package
Requires:       %{name}    == %{version}-%{release}

Requires:       coreutils  >= %{coreutils_evr}

Conflicts:      coreutils          < %{coreutils_evr}
Conflicts:      coreutils-single

%description    checksum
Provides drop-in replacements for sha*sum tools (from package
coreutils) using %{name}.
%endif


%if %{with replace_fipscheck}
%package        fipscheck
Summary:        Drop-in replacements for fipscheck/fipshmac provided by the %{name} package
Requires:       %{name}   == %{version}-%{release}

Obsoletes:      fipscheck         <= %{fipscheck_evr}

Provides:       fipscheck         == %{fipscheck_evr}.1

%description    fipscheck
Provides drop-in replacements for fipscheck and fipshmac tools (from
package fipscheck) using %{name}.
%endif


%if %{with replace_hmaccalc}
%package        hmaccalc
Summary:        Drop-in replacements for hmaccalc provided by the %{name} package
Requires:       %{name}   == %{version}-%{release}

Obsoletes:      hmaccalc          <= %{hmaccalc_evr}

Provides:       hmaccalc          == %{hmaccalc_evr}.1

%description    hmaccalc
Provides drop-in replacements for sha*hmac tools (from package
hmaccalc) using %{name}.
%endif


%package        static
Summary:        Static library for -static linking with %{name}
Requires:       %{name}-devel == %{version}-%{release}

%description    static
This package contains the %{name} static libraries for -static
linking.  You don't need this, unless you link statically, which
is highly discouraged.


%package        tools
Summary:        Utility applications for the %{name} package
Requires:       %{name} == %{version}-%{release}

%description    tools
Utility applications that are provided with %{name}.  This includes
tools to use message digests, symmetric ciphers and random number
generators implemented in the Linux kernel from command line.


%if %{with test_package}
%package        tests
Summary:        Testing scripts for the %{name} package
Requires:       %{name}       == %{version}-%{release}
Requires:       %{name}-tools == %{version}-%{release}
%if %{with replace_hmaccalc}
Requires:       %{name}-hmaccalc == %{version}-%{release}
%endif
%if %{with replace_coreutils}
Requires:       %{name}-checksum == %{version}-%{release}
%endif
Requires:       coreutils
Requires:       openssl
Requires:       perl-interpreter

%description    tests
Auxiliary scripts for testing %{name}.
%endif

%if %{with doc}
%package        doc
Summary:        User documentation for the %{name} package
BuildArch:      noarch
Requires:       %{name} == %{version}-%{release}

%description    doc
User documentation for %{name}.
%endif


%prep
%autosetup -p1

%build
autoreconf -fiv

%configure               \
  --libdir=/%{_lib}      \
  --disable-silent-rules \
  --enable-kcapi-encapp  \
  --enable-kcapi-dgstapp \
  --enable-kcapi-hasher  \
  --enable-kcapi-rngapp  \
  --enable-kcapi-speed   \
  --enable-kcapi-test    \
  --enable-shared        \
  --enable-static        \
  --enable-sum-prefix=   \
  --enable-sum-dir=/%{_lib} \
  --with-pkgconfigdir=%{_libdir}/pkgconfig
%if %{with doc}
%make_build all doc
%else
%make_build all man
%endif


%install
%make_install

mkdir -p %{buildroot}%{_sysctldir}
install -Dpm 0644 -t %{buildroot}%{_sysctldir} %{SOURCE5}

mkdir -p %{buildroot}%{_pkgdocdir}
install -Dpm 0644 -t %{buildroot}%{_pkgdocdir} \
%if %{with_sysctl_tweak}
  %{SOURCE4}                                        \
%endif
%if %{with doc}
  doc/%{name}.p{df,s}                               \
%endif
  README.md CHANGES.md TODO

%if %{with doc}
cp -pr lib/doc/html %{buildroot}%{_pkgdocdir}
%endif

%if !%{with replace_coreutils}
rm -f                            \
  %{buildroot}%{_bindir}/md5sum       \
  %{buildroot}%{_bindir}/sha*sum      \
  %{buildroot}%{_bindir}/sm*sum
%endif

%if !%{with replace_fipscheck}
rm -f %{buildroot}%{_bindir}/fips*
%endif

%if !%{with replace_hmaccalc}
rm -f %{buildroot}%{_bindir}/sha*hmac
rm -f %{buildroot}%{_bindir}/sm*hmac
%endif

find %{buildroot} -type f -name '*.la' -print -delete
find %{buildroot} -type f -name '*.hmac' -print -delete
find %{buildroot} -type f -size 0 -print -delete

%if %{with doc}
find %{buildroot}%{_pkgdocdir} -type f -print | xargs %{__chmod} -c 0644
find %{buildroot}%{_pkgdocdir} -type d -print | xargs %{__chmod} -c 0755
%endif

for d in %{_mandir} %{_pkgdocdir}; do
  %{_bindir}/hardlink -cfv %{buildroot}$d
done


%generate_compatibility_deps

%check
%if %{with clang_sa}
%make_build scan
%endif
%if %{with cppcheck}
%make_build cppcheck
%endif

%if %{with test}
%if %(cat /proc/sys/net/core/optmem_max || echo 0) >= 20480
%if %{lua:print(rpm.vercmp(posix.uname('%r'), '5.1'));} >= 0
pushd test
%if %{with fuzz_test}
ENABLE_FUZZ_TEST=1 \
%endif
NO_32BIT_TEST=1    \
  ./test-invocation.sh
popd
%endif
%endif
%endif

%ldconfig_scriptlets


%files
%doc %dir %{_pkgdocdir}
%doc %{_pkgdocdir}/README.md
%license COPYING*
%dir %{abidir}
/%{_lib}/%{name}.so.*
%{abidir}/%{name}.dump
/%{_lib}/fipscheck/%{name}.so.*
%if %{with_sysctl_tweak}
%doc %{_pkgdocdir}/README.%{distroname_ext}
%{_sysctldir}/%{sysctl_prio}-%{name}-optmem_max.conf
%endif


%files          devel
%doc %{_pkgdocdir}/CHANGES.md
%doc %{_pkgdocdir}/TODO
%{_includedir}/kcapi.h
%{_mandir}/man3/kcapi_*.3.*
/%{_lib}/%{name}.so
%{_libdir}/pkgconfig/%{name}.pc


%if %{with replace_coreutils}
%files          checksum
%{_bindir}/md5sum
%{_bindir}/sha*sum
%{_bindir}/sm*sum
/%{_lib}/fipscheck/md5sum.hmac
/%{_lib}/fipscheck/sha*sum.hmac
/%{_lib}/fipscheck/sm*sum.hmac
%endif

%if %{with replace_fipscheck}
%files          fipscheck
%{_bindir}/fips*
/%{_lib}/fipscheck/fips*.hmac
%endif

%if %{with replace_hmaccalc}
%files          hmaccalc
%{_bindir}/sha*hmac
%{_bindir}/sm*hmac
/%{_lib}/hmaccalc/sha*hmac.hmac
/%{_lib}/hmaccalc/sm*hmac.hmac
%endif


%files          static
/%{_lib}/%{name}.a


%files          tools
%{_bindir}/kcapi*
%{_mandir}/man1/kcapi*.1.*


%if %{with test_package}
%files          tests
%{_libexecdir}/%{name}/*
%endif


%if %{with doc}
%files          doc
%doc %{_pkgdocdir}/html
%doc %{_pkgdocdir}/%{name}.pdf
%doc %{_pkgdocdir}/%{name}.ps
%endif


%changelog
* Tue Apr 11 2023 Ziyang Zhang <ZiyangZhang@linux.alibaba.com> -1.4.0-2
- Optimize spec file

* Thu Sep 22 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.4.0-1
- update to 1.4.0

* Wed Mar 09 2022 DengXiewei <dengxiewei@linux.alibaba.com> - 1.3.1-1
- Init from upstream 1.3.1
